/*
 * Rahmenwerk-Plug-in "Anlagenstatus"
 * Copyright (C) 2010 BitCtrl Systems GmbH
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.anlagenstatus.internal;

import java.util.ArrayList;
import java.util.List;

import de.bsvrz.buv.rw.basislib.berechtigung.IOberflaechenBerechtigungZuhoerer;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigung;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenBerechtigungEreignis;
import de.bsvrz.buv.rw.basislib.berechtigung.OberflaechenFunktion;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.DaVPreLader;
import de.bsvrz.buv.rw.rw.dav.datenbasis.prelader.IDaVPreLader;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Zentrale Instanz zur Verwaltung der Berechtigungen fr die Funktionen des
 * Benutzerverwaltung/Zugriffsrechte-Plugins.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker, Steffen Gieseler
 * 
 * @version $Id: AnlagenStatusBerechtigungsVerwaltung.java 23300 2010-05-25 16:13:37Z gieseler $
 */
public final class AnlagenStatusBerechtigungsVerwaltung implements
		IOberflaechenBerechtigungZuhoerer, IDaVPreLader {

	/**
	 * Debug-Logger.
	 */
	private final Debug debug = Debug.getLogger();

	/** das Singleton-Objekt fr die Verwaltung der Berechtigungen. */
	private static final AnlagenStatusBerechtigungsVerwaltung INSTANCE = new AnlagenStatusBerechtigungsVerwaltung();

	/**
	 * liefert die Instanz des Verwaltungsobjekts.
	 * 
	 * @return das Objekt
	 */
	public static IOberflaechenBerechtigungZuhoerer getInstanz() {
		return AnlagenStatusBerechtigungsVerwaltung.INSTANCE;
	}

	/**
	 * die Liste der Objekte, fr die die Berechtigungen abgefragt werden.
	 * 
	 * Das Berechtigungskonzept des Rahmenwerks erfordert zur Prfung von
	 * Berechtigungen in jedem Fall die bergabe eines Systemobjekts, auch wenn
	 * die Berechtigung an sich nicht objektbezogen ist. Fr die globale
	 * Verwaltung der UDA-Pluginberechtigungen wird deshalb hier das lokale
	 * Applikationsobjekt in die Liste eingetragen und bei der Prfung der
	 * Berechtigungen bergeben.
	 */
	private final List<SystemObject> berechtigunsObjektListe = new ArrayList<SystemObject>();

	/** Konstruktor. */
	private AnlagenStatusBerechtigungsVerwaltung() {
		super();
		DaVPreLader.getInstanz().hinzufuegenListenerPreLader(this);
		if (berechtigunsObjektListe.isEmpty()) {
			berechtigunsObjektListe.add(DaVVerbindung.getInstanz()
					.getDaVVerbindung().getLocalApplicationObject());
		}
		OberflaechenBerechtigung.getInstanz().pruefeBerechtigungElementGruppe(
				berechtigunsObjektListe, PluginAnlagenStatus.class.getName());
	}

	/**
	 * {@inheritDoc}
	 * 
	 * Die Funktion verarbeitet das Ereignis, mit dem genderte Bwerechtigungen
	 * fr Funktionen und Objekte und setzt Berechtigungszustand der
	 * angemeldeten Uda-Plugin-Funktionen.
	 */
	@Override
	public void freigabe(final OberflaechenBerechtigungEreignis e) {

		debug.config("Benutzer-Plugin Freigabe empfangen");
		List<OberflaechenFunktion> liste = e.getOberflaechenFunktionSperrung();
		if (liste.size() == 0) {
			debug.config("keine gesperrten Elemente");
		} else {
			debug.config(liste.size() + " gesperrte Elemente");
		}
		for (OberflaechenFunktion oFunk : liste) {
			final AnlagenStatusPluginFunktion funktion = AnlagenStatusPluginFunktion
					.getFunktion(oFunk.getFunktionsName());
			if (funktion != null) {
				funktion.setFreigegeben(false);
			}
		}

		liste = e.getOberflaechenFunktionFreigabe();
		if (liste.size() == 0) {
			debug.config("keine freigegebenen Elemente");
		} else {
			debug.config(liste.size() + " freigegebene Elemente");
		}
		for (OberflaechenFunktion oFunk : liste) {
			final AnlagenStatusPluginFunktion funktion = AnlagenStatusPluginFunktion
					.getFunktion(oFunk.getFunktionsName());
			if (funktion != null) {
				funktion.setFreigegeben(true);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public void sperrung(final OberflaechenBerechtigungEreignis e) {
		freigabe(e);
	}

	/** {@inheritDoc} */
	@Override
	public void systemObjekteAktualisierung(final GtmDatenBasis datenBasis) {
		// keine Aktionen erforderlich
	}

	/**
	 * {@inheritDoc}
	 * 
	 * die initiale Belegung der Berechtigung der Funktionen des UDA-Plugins
	 * wird abgefragt, wenn der DavPrelader initialisiert ist, da vorher keine
	 * sinnvolle Abfrage mglich ist.
	 * 
	 * <b>Bekannter Fehler: Die Funktion
	 * <code>istInitialesLadenAbgeschlossen</code> des DavPrelader liefert
	 * <code>false</code>, obwohl das Ende des initialen Ladevorganges gemeldet
	 * wurde!</b>
	 */
	@Override
	public void systemObjekteInitial(final GtmDatenBasis datenBasis) {
		// System.err.println("Prelader initialisiert = "
		// + DaVPreLader.getInstanz().istInitialesLadenAbgeschlossen());

		if (DaVPreLader.getInstanz().istInitialesLadenAbgeschlossen()) {
			if (berechtigunsObjektListe.isEmpty()) {
				berechtigunsObjektListe.add(DaVVerbindung.getInstanz()
						.getDaVVerbindung().getLocalApplicationObject());
			}
			OberflaechenBerechtigung.getInstanz()
					.pruefeBerechtigungElementGruppe(berechtigunsObjektListe,
							PluginAnlagenStatus.class.getName());
			debug
					.config("Anlagenstatus-Plugin PreLader-Initialisierung abgeschlossen");
		}
	}
}
